
let scenarioWorld = require('@cellebrite/automation-wdio-helpers/scenarioWorld');
import { sendRequest } from '../common/rest-api/sendRequest';

const logger = require('../../node_modules/@cellebrite/automation-loggers/loggerModule');
const userQuery = require('../api_objects/Queries/user');
const postOptionsTemplate = require('../api_objects/Queries/postOptions');

import { expect } from 'chai';
declare const global: any;

export class appPage {
    constructor() { };

    appId: string;
    appKey: string;

    static async createAppByAPI(appName: string) {
        logger.verbose("Creating app...");
        let jsonObject = userQuery.query;

        jsonObject.name = appName;
        jsonObject.description = "fake description";
        jsonObject.key = "fake key";
        let postOptions = postOptionsTemplate.post;
        postOptions.uri = global.runSettings.umsUrl + '/v1/applications';

        logger.verbose("Sending request for creating application");
        let Res = await sendRequest.sendRequest(jsonObject, postOptions);
        logger.verbose("request sent");
        if (Res.statusCode == 201) {
            scenarioWorld.appId = Res.body.id;
            scenarioWorld.appKey = Res.body.key;
        }

        return Res;
    }
    static async isAppExistByName(appName: string) {
        logger.debug("checking user " + appName + "exist...");
        let appList = await this.getAppList()
        let appExist = this.checkAppExistByName(appList, appName);
        return appExist;
    }
    static async getAppId(appName: string) {
        logger.verbose("getting app " + appName + "Id");
        let appList = await this.getAppList();
        return this.getIdbyApp(appList, appName);
    }
    static async getAppKey(appName: string) {
        logger.verbose("getting app " + appName + "Key");
        let appList = await this.getAppList();
        return this.getKeybyApp(appList, appName);
    }
    static async updateApp(appId: string, appName: string) {
        logger.verbose("updating application...");
        let jsonObject = userQuery.query;

        jsonObject.name = "updatedapp" + global.timestamp;
        jsonObject.description = "updated description" + global.timestamp;


        let postOptions = postOptionsTemplate.put;
        postOptions.uri = global.runSettings.umsUrl + '/v1/applications/' + appId;
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;

        logger.verbose("Sending request for updating application");
        let profileRes = await sendRequest.sendRequest(jsonObject, postOptions)
        logger.verbose("request sent");
        return profileRes;
    }
    static async getAppList() {
        let getOptions = postOptionsTemplate.get;
        getOptions.uri = global.runSettings.umsUrl + '/v1/applications';
        getOptions.resolveWithFullResponse = false;
        let appList = await sendRequest.sendRequest("{}", getOptions);
        logger.debug('request sent');
        return appList;
    }
    static checkAppExistByName(appList: any, appName: string) {
        let i;
        for (i = 0; i < appList.length; i++) {
            if (appList[i].name == appName)
                return true;
        }
        return false;
    }
    static getIdbyApp(appList: any, appName: string) {
        let i;
        for (i = 0; i < appList.length; i++) {
            if (appList[i].name == appName)
                return appList[i].id;
        }
        return -1;
    }
    static getKeybyApp(appList: any, appName: string) {
        let i;
        for (i = 0; i < appList.length; i++) {
            if (appList[i].name == appName)
                return appList[i].key;
        }
        return -1;
    }
    static async initApp(appName: string) {
        let res: any = await this.isAppExistByName(appName);
        if (!res) {
            res = await this.createAppByAPI(appName);
            if (!(res.statusCode == 201)) {
                expect(false, `Error: ${appName} application was not created!`).to.be.true;
            }
        }
    }
}