import {sendRequest} from '../common/rest-api/sendRequest';
let scenarioWorld = require('@cellebrite/automation-wdio-helpers/scenarioWorld');
const logger = require('@cellebrite/automation-loggers/loggerModule');
const postOptionsTemplate = require('../api_objects/Queries/postOptions');
import {expect} from 'chai';

declare const global : any;
declare const constants : any;
export class groupPage {
    constructor(){};
    static async createGroupByAPI(groupName : string, roleId : string){
        logger.verbose("Creating group...");
        let jsonObject = constants["group request"];
        jsonObject.name = groupName;
        jsonObject.roles = [{id: roleId}];
        jsonObject.description = "fakegroup";
        let postOptions = postOptionsTemplate.post;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/groups';
    
        logger.verbose("Sending request for creating group");
        let profileRes = await sendRequest.sendRequest(jsonObject,postOptions)
        logger.verbose("request sent");
        return profileRes;
    }
    static async isGroupExistByName(GroupName : string){
        
        logger.debug("checking group " + GroupName +   "exist...");
        let groupList = await this.getGroupsList()
        let groupExist = this.checkGroupExistByName(groupList, GroupName);
        return groupExist;
    
    }
    static async getGroupsList(){
        let getOptions = postOptionsTemplate.get;    
        getOptions.uri = global.runSettings.umsUrl + '/v1/groups';
        getOptions.resolveWithFullResponse = false;
        //delete getOptions.body;
        let groupsList = await sendRequest.sendRequest("{}",getOptions);
        logger.debug('request sent');
        return groupsList;
    }
    static async getGroupId(groupName : string){
        logger.verbose("getting group " + groupName  + " Id");
        let groupList = await this.getGroupsList();
        return this.getIdbyGroup(groupList,groupName);
    }
    static async findGroupById(groupId : string){
        let jsonObject ={};
        let postOptions = postOptionsTemplate.get;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/groups/' + groupId;
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;
        let res = await sendRequest.sendRequest(jsonObject,postOptions);
        logger.verbose("send request to find group by Id: " + groupId);
        return res;
    }
    static async updateGroup(groupId : string, groupName : string){
        logger.verbose("updating group..." + groupName);
        let jsonObject = constants["group request"];
          
        jsonObject.name = "updatedGroupName" + global.timestamp;
        
        let postOptions = postOptionsTemplate.put;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/groups/' + groupId;
        logger.verbose("URI: " + postOptions.uri);
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;
    
        logger.verbose("Sending request for updating group");
        let res = await sendRequest.sendRequest(jsonObject,postOptions)
        logger.verbose("request sent");
        if(res.statusCode !== 200){
            logger.verbose("the error is: " + JSON.stringify(res.body));
            
        }
        
        return res;
    }
    static async getRolesByGroupId(groupId : string){
        let getOptions = postOptionsTemplate.get;    
        getOptions.uri = global.runSettings.umsUrl + '/v1/groups/' + groupId + "/roles";
        getOptions.resolveWithFullResponse = false;
        let rolesList = await sendRequest.sendRequest("{}",getOptions);
        logger.debug('request sent');
        return rolesList;
    }
    static async addRolesToGroup(roleId : string, groupId : string){
        let getOptions = postOptionsTemplate.post;
        let jsonObject = constants["defaults request"];
        jsonObject = {
            "roles": [
              {
                "id": roleId
              }
            ]
          }
        getOptions.uri = global.runSettings.umsUrl + '/v1/groups/' + groupId + "/roles";
        getOptions.resolveWithFullResponse = true;
        let res = await sendRequest.sendRequest(jsonObject,getOptions);
        logger.debug('request sent');
        return res;
    }
    static async updateRoleToGroup(roleId : string, groupId : string){
        let getOptions = postOptionsTemplate.put;
        let jsonObject = constants["defaults request"];
        jsonObject = {
            "roles": [
              {
                "id": roleId
              }
            ]
          }
        getOptions.uri = global.runSettings.umsUrl + '/v1/groups/' + groupId + "/roles";
        getOptions.resolveWithFullResponse = true;
        let res = await sendRequest.sendRequest(jsonObject,getOptions);
        logger.debug('request sent');
        return res;
    }
    static async addUserToGroup(groupId : string, userId : string){
        let getOptions = {            
            method: 'POST',
            uri: '<Mandatory>',
            headers: {        
                        
            },
            json: true,
            simple: false,
            resolveWithFullResponse: true, 
        };
        let jsonObject = {
            "users": [
              {
                "id": userId
              }
            ]
          }
        getOptions.uri = global.runSettings.umsUrl + '/v1/groups/' + groupId + "/users";
        getOptions.resolveWithFullResponse = true;
        let res = await sendRequest.sendRequest(jsonObject,getOptions);
        logger.debug('request sent');
        return res;
    }
    static async initGroup(groupName : string, roleId : string){
        let res = await this.isGroupExistByName(groupName);
        if(!res){
            await this.createGroupByAPI(groupName, roleId)
            res = await this.isGroupExistByName(groupName);
            if(!res){
                expect(false, `Error: ${groupName} group was not created!`).to.be.true;
            }
        }                    
    }
    //##############################################################################################################################################################
    static checkGroupExistByName(groupList : any, groupName : string){
        let i;
        for(i = 0;i < groupList.length; i++){
            if(groupList[i].name == groupName)
                return true;
        }
        return false;
    }
    static getIdbyGroup(groupList : any, groupName : string){
        let i;
        for(i = 0; i < groupList.length; i++){
            if(groupList[i].name == groupName)
                return groupList[i].id;
        }
        return -1;
    }
}
