import {sendRequest} from '../common/rest-api/sendRequest';
let scenarioWorld = require('@cellebrite/automation-wdio-helpers/scenarioWorld');
const logger = require('@cellebrite/automation-loggers/loggerModule');
const postOptionsTemplate = require('../api_objects/Queries/postOptions');
import {expect} from 'chai';

declare const global : any;
export class permissionPage {
    permId :string ;
    static async isPermExistByName(permName : string){
        logger.debug("checking perm " + permName +   "exist...");
        let permList = await this.getPermList()
        let permExist = this.checkPermExistByName(permList, permName);
        return permExist;
    }
    static async getAllPermission(appKey : any, userName : string){
        let jsonObject : any = {
            'json': true
        };
        let postOptions = postOptionsTemplate.get; 
        postOptions.uri = global.runSettings.umsUrl + `/v1/permissions/application/${appKey}?username=${userName}`;
        let res = await sendRequest.sendRequest(jsonObject,postOptions);
        return res;
    }
    static async getPermList(){
        let getOptions = postOptionsTemplate.get;    
        getOptions.uri = global.runSettings.umsUrl + '/v1/permissions';
        getOptions.resolveWithFullResponse = false;
        let permList = await sendRequest.sendRequest("{}",getOptions);
        logger.debug('request sent');
        return permList;
    }
    static async createPermByAPI(permName : string, appId : string, appKey : string){
        logger.verbose("Creating permission...");
        let jsonObject : any = {
            'json': true
        };
        let myarr : any = [];
        jsonObject.name = permName;
        jsonObject.description = "fake description";
        jsonObject.applicationId = appId;
        jsonObject.applicationKey = appKey;
        myarr.push(jsonObject); 
        let postOptions = postOptionsTemplate.post;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/permissions';
    
        logger.verbose("Sending request for creating permissions");
        let Res = await sendRequest.sendRequest(myarr,postOptions)
        logger.verbose("request sent");
        if(Res.statusCode == 201){
            scenarioWorld.permId = Res.body[0].id;
        }
        return Res;
    }
    static async getPermId(permName : string){
        logger.verbose("getting user " + permName + "Id");
        let permList = await this.getPermList();
        return this.getIdbyPerm(permList,permName);
    }
    static async updatePerm(permId : string, appId : string, appKey : string){
        
        logger.verbose("updating application...");
        let jsonObject : any = {
            'json': true
        };
          
        jsonObject.name = "updatedperm" + global.timestamp;
        jsonObject.description = "updated description" + global.timestamp;
        jsonObject.applicationId = appId;
        jsonObject.applicationKey = appKey;
        
        
        let postOptions = postOptionsTemplate.put;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/permissions/' + permId;
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;
    
        logger.verbose("Sending request for updating permissions");
        let profileRes = await sendRequest.sendRequest(jsonObject,postOptions)
        logger.verbose("request sent");
        return profileRes;
    }
    static checkPermExistByName(permList : any, permName : string){
        let i;
        for(i = 0;i < permList.length; i++){
            if(permList[i].name == permName)
                return true;
        }
        return false;
    }
    static getIdbyPerm(permList : any, permName : string){
        let i;
        for(i = 0; i < permList.length; i++){
            if(permList[i].name == permName)
                return permList[i].id;
        }
        return -1;
    }
    static async initPerm(permName : string, appId : string, appKey :string){
        let res :any = await this.isPermExistByName(permName);
                if(!res){
                    res = await this.createPermByAPI(permName, appId, appKey);
                    if(!(res.statusCode == 201)){
                        expect(false, `Error: ${permName} permission was not created!`).to.be.true;
                    }
                }
    }
    
}