const logger = require('../../node_modules/@cellebrite/automation-loggers/loggerModule');
const userQuery = require('./Queries/user');
const postOptionsTemplate = require('./Queries/postOptions');
import {sendRequest} from '../common/rest-api/sendRequest';
import { query } from 'winston';
import { profile } from 'winston';
import { json } from 'body-parser';
let scenarioWorld = require('@cellebrite/automation-wdio-helpers/scenarioWorld');

declare const global : any;

export class profilePage {
    userId : string ;
    public static async createUserByAPI(userName: string, user : any) {
        logger.verbose("Creating user...");
       let jsonObject : any = {
            username: userName,
            ...user.firstName && {firstName: user.firstName + global.timestamp},
            ...user.lastName && {lastName: user.lastName + global.timestamp},
            ...user.password && {password: user.password},
            ...user.email && {email: user.email},
            ...user.status && {status: user.status},
            ...user.phone && {phone: user.phone},
            ...user.department && {department: user.department},
            ...user.location && {location: user.location}
        }
        
        let postOptions = postOptionsTemplate.post;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/users';
    
        logger.verbose("Sending request for creating user");
        let profileRes = await sendRequest.sendRequest(jsonObject,postOptions)
        logger.verbose("request sent");
        if(profileRes.statusCode == 201){
            scenarioWorld.userId = profileRes.body.id;
        }
        return profileRes;
    }
    static async getUserId(userName : string){
        logger.verbose("getting user " + userName + "Id");
        let userList = await profilePage.getUsersList();
        return profilePage.getIdbyUser(userList,userName);
    }
    static async setPassword(password: string,userId: string){
        let jsonObject ={password: password};
        let postOptions = postOptionsTemplate.post;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/users/' + userId + '/password';
        let res = await sendRequest.sendRequest(jsonObject,postOptions);
        logger.verbose("send request for setting new password: " + password + " to userId: " + userId);
        return res.body;
    }
    static async isUserExistByFirstName(userName: string){
        logger.debug("checking user " + userName +   " exist...");
        let userList = await profilePage.getUsersList()
        let userExist = this.checkUserExistByFirstName(userList, userName);
        return userExist;
    }
    static async findUserById(userId: string){
        let jsonObject ={};
        let postOptions = postOptionsTemplate.get;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/users/' + userId;
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;
        let res = await sendRequest.sendRequest(jsonObject,postOptions);
        logger.verbose("send request to find user by Id: " + userId);
        return res;
    }
    static async updateUser(userId: string, userName: string, user : any){
        logger.verbose("updating user...");
        
        let jsonObject : any = {
            ...user.firstName && {firstName: user.firstName + global.timestamp},
            ...user.username && {username: userName },
            ...user.lastName && {lastName: user.lastName + global.timestamp},
            ...user.password && {password: user.password},
            ...user.email && {email: user.email},
            ...user.status && {status: user.status},
            ...user.phone && {phone: user.phone},
            ...user.department && {department: user.department},
            ...user.location && {location: user.location},
        }
        if(user.roles){
            let roleId = [{id: user.roles.id}]
            jsonObject.roles = roleId;
        }
        
        let postOptions = postOptionsTemplate.put;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/users/' + userId;
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;
    
        logger.verbose("Sending request for updating user");
        let profileRes = await sendRequest.sendRequest(jsonObject,postOptions)
        logger.verbose("request sent");
        return profileRes;
    }
    static async deleteUserById(userId: string){
        let jsonObject ={};
        let postOptions = postOptionsTemplate.delete;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/users/' + userId;
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;
        let res = await sendRequest.sendRequest(jsonObject,postOptions);
        logger.verbose("send request to find user by Id: " + userId);
        return res;
    }
    static async getUsersList(){
        let getOptions = postOptionsTemplate.get;    
        getOptions.uri = global.runSettings.umsUrl + '/v1/users';
        getOptions.resolveWithFullResponse = false;
        //delete getOptions.body;
        let userList = await sendRequest.sendRequest("{}",getOptions);
        logger.debug('request sent');
        return userList;
    }
    static async getUsersListByStatus(userStatus : string){
        let getOptions = postOptionsTemplate.get;    
        getOptions.uri = global.runSettings.umsUrl + `/v1/users?status=${userStatus}`;
        getOptions.resolveWithFullResponse = false;
        let userList = await sendRequest.sendRequest("{}",getOptions);
        logger.debug('request sent');
        return userList;
    }
    static async searchUsers(searchString : string){
        let getOptions = postOptionsTemplate.get;    
        getOptions.uri = global.runSettings.umsUrl + `/v1/users?search=${searchString}`;
        getOptions.resolveWithFullResponse = false;
        let userList = await sendRequest.sendRequest("{}",getOptions);
        logger.debug('request sent');
        return userList;
    }
    static getIdbyUser(userList: any, userName: string){
        let i;
        for(i = 0; i < userList.length; i++){
            if(userList[i].externalId == userName)
                return userList[i].id;
        }
        return -1;
    }
    
    static checkUserExistByFirstName(userList : any, firstName : string){
        let i;
        for(i = 0;i < userList.length; i++){
            if(userList[i].externalId == firstName)
                return true;
        }
        return false;
    }
    static checkUsersStatus(userList : any, userStatus : string){
        let i;        
        for(i = 0;i < userList.length; i++){
            if(userList[i].status !== userStatus)
                return false;
        }
        return true;
    }
}