import {sendRequest} from '../common/rest-api/sendRequest';
let scenarioWorld = require('@cellebrite/automation-wdio-helpers/scenarioWorld');
const logger = require('@cellebrite/automation-loggers/loggerModule');
const userQuery = require('./Queries/user');
const postOptionsTemplate = require('./Queries/postOptions');
import {expect} from 'chai';

declare const global : any;
declare const constants : any;

export class rolePage {
    //constructor(){};
    roleId : string;
    static async createRoleByAPI(roleName : string, permId : string, predefined :any = undefined) {
        logger.verbose("Creating role...");
        //let jsonObject = userQuery.query;
        let jsonObject = constants["role request"];
        let myarr : any = [];
        jsonObject.name = roleName;
        
        jsonObject.permissions = [{id: permId}];
        if(predefined){
            jsonObject.predefined = predefined;
        }
        
        jsonObject.description = "fakerole";
        let postOptions = postOptionsTemplate.post;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/roles';
        
        myarr.push(jsonObject);
        logger.verbose("Sending request for creating role");
        let Res = await sendRequest.sendRequest(myarr,postOptions)
        logger.verbose("request sent");
        if(Res.statusCode == 201){
            scenarioWorld.roleId = Res.body[0].id;
        }
        return Res;
        
    }
    static async isRoleExistByName(roleName : string){
        
            logger.debug("checking role " + roleName +   "exist...");
            let rolesList = await this.getRolesList()
            let roleExist = this.checkRoleExistByName(rolesList, roleName);
            return roleExist;
        
    }
    static async getRolesList(){
        let getOptions = postOptionsTemplate.get;    
        getOptions.uri = global.runSettings.umsUrl + '/v1/roles';
        getOptions.resolveWithFullResponse = false;
        let rolesList = await sendRequest.sendRequest("{}",getOptions);
        logger.debug('request sent');
        return rolesList;
    }
    static async getRolesListWithPermissions(roleId : number){
        let getOptions = postOptionsTemplate.post;    
        getOptions.uri = global.runSettings.umsUrl + '/v1/roles/permissions/search';
        getOptions.resolveWithFullResponse = false;
        let jsonObject : any  = {};
        jsonObject = {
            "ids": [
              roleId
            ]
          }
        let rolesWithPermissions = await sendRequest.sendRequest(jsonObject,getOptions);
        logger.debug('request sent');
        return rolesWithPermissions;
    };
    static async getRoleId(roleName : string){
        logger.verbose("getting user " + roleName + "Id");
        let roleList = await this.getRolesList();
        return this.getIdbyRole(roleList,roleName);
    }
    static async updateRole(roleId : string, permId : string){
        logger.verbose("updating Role...");
        let jsonObject : any = {
            'json': true
        };
          
        jsonObject.name = "updatedrole" + global.timestamp;
        jsonObject.description = "updated description" + global.timestamp;
        jsonObject.permissions = [{id: permId}];
        
        
        let postOptions = postOptionsTemplate.put;    
        postOptions.uri = global.runSettings.umsUrl + '/v1/roles/' + roleId;
        postOptions.resolveWithFullResponse = true;
        postOptions.simple = false;
    
        logger.verbose("Sending request for updating role");
        let profileRes = await sendRequest.sendRequest(jsonObject,postOptions)
        logger.verbose("request sent");
        return profileRes;
    }
    
    static getIdbyRole(roleList : any, roleName : string){
        let i;
        for(i = 0; i < roleList.length; i++){
            if(roleList[i].name == roleName)
                return roleList[i].id;
        }
        return -1;
    }
    static checkRoleExistByName(rolesList : any, roleName : string) : boolean{
        let i;
        for(i = 0;i < rolesList.length; i++){
            if(rolesList[i].name == roleName)
                return true;
        }
        return false;
    }
    static async initRole(roleName : string, permId : string){
        let res : any = await this.isRoleExistByName(roleName);
        if(!res){
            res = await this.createRoleByAPI(roleName, permId);
            if(!(res.statusCode == 201)){
                expect(false, `Error: ${roleName} role was not created!`).to.be.true;
            }
        }
    }
}