
rp = require('request-promise').defaults({ jar: true });

async function authenticateUser(username, password, grant_type, client_id, client_secret, scope, response_type, issuer, redirect_uri) {
    
    var j = rp.jar();
    rp = rp.defaults({jar: j});
    const grantResponse = await requestGrant(client_id, scope, response_type, issuer);
    const grant = grantResponse.headers.location.match(`\/interaction\/(.*)`)[1];
    let code = "";
    await requestLogin(grant, username, password, issuer);
    const codeResponse = await requestCode(grant, client_id, scope, response_type, issuer);
    try {
        code = codeResponse.headers.location.match(`code=([^&]*)&?`)[1];
    } catch (err) {
        return null;
    }
    const tokenSet = await requestToken(code, grant_type, client_id, client_secret, issuer, redirect_uri);
    return tokenSet.access_token;
}

async function requestGrant(client_id, scope, response_type, issuer) {
    return rp({
        uri: `${issuer}/oidc/auth`, strictSSL: false, simple: false, followRedirect: false, resolveWithFullResponse: true,
        qs: {
            client_id, scope, response_type
        }
    });
}

async function requestCode(grant, client_id, scope, response_type, issuer) {
    return rp({
        uri: `${issuer}/oidc/auth/${grant}`, strictSSL: false, simple: false, followRedirect: false, resolveWithFullResponse: true,
        qs: {
            client_id, scope, response_type
        }
    });
}

async function requestLogin(grant, username, password, issuer) {
    return rp({
        uri: `${issuer}/interaction/${grant}/login`, strictSSL: false, simple: false,
        method: 'POST',
        form: {
            login: username,
            password: password
        }
    });
}

async function requestToken(code, grant_type, client_id, client_secret, issuer, redirect_uri) {
    return rp({
        uri: `${issuer}/oidc/token`, strictSSL: false, simple: false, json: true,
        method: 'POST',
        form: {
            grant_type, client_id, client_secret, code, redirect_uri
        }
    });
}





module.exports.authenticateUser = authenticateUser;