
import { defineSupportCode } from "cucumber";
import { sendRequest } from "../../common/rest-api/sendRequest";
const scenarioWorld = require("../../../node_modules/@cellebrite/automation-wdio-helpers/scenarioWorld");
import { expect } from "chai";
let rp = require("request-promise");
import { profilePage } from "../../api_objects/profilePage";
const updateJson = require("../../../node_modules/@cellebrite/automation-common-helpers/updateJson");
let jsonDiff = require("json-diff");
const logger = require("@cellebrite/automation-loggers/loggerModule");

declare const constants: any;
declare const global: any;

defineSupportCode(({ Given, When, Then }) => {

    Given(/^the request is: (.*)* "(.*)"$/,
        function(reqName, request) {
            this.requestJson = constants[request];
            logger.verbose("the request name is: " + reqName);
        });

    Given(/^the request is:$/,
        function(json) {
            if (json !== undefined && json !== "") {
                this.requestJson = JSON.parse(json);
            }
        });

    When(/^I (post|put|get|delete) to "(.*)"$/,
        async function(method, uri) {
            this.requestJson.resolveWithFullResponse = true;
            this.requestJson.method = method;
            this.requestJson.uri = global.runSettings.umsUrl + uri;
            this.requestJson.simple = false;
            this.responseJson = await sendRequest.sendRequest({}, this.requestJson);
        });

    Then(/^response status code should( not)* be (\d+)$/,
        function(falseCase, statusCode) {
            const actualStatusCode = this.responseJson.statusCode;
            const errorMsg = "Error in http status code";
            if (falseCase) {
                expect(actualStatusCode, errorMsg).to.not.equal(statusCode);
            } else {
                expect(actualStatusCode, errorMsg).to.equal(statusCode);
            }
        });

    Given(/^I create user name "([^"]*)"$/,
        async function(userName) {
            this.uniqueUserName = userName + global.timestamp;
            let user = constants[userName];
            if (!user) {
                user = {};
            }
            const res = await profilePage.isUserExistByFirstName(this.uniqueUserName);
            if (!res) {
                await profilePage.createUserByAPI(this.uniqueUserName, user);
            }
        });

    Given(/^user "([^"]*)" exist$/,
        async function(userName) {
            const res = await profilePage.isUserExistByFirstName(userName);
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error: ${userName} does not exist!`).to.be.true;
        });

    When(/^I change "([^"]*)" status to "([^"]*)"$/,
        async function(userName, status) {
            userName = userName + global.timestamp;
            const user = { status };
            const userId = scenarioWorld.userId;
            this.responseJson = await profilePage.updateUser(userId, userName, user);
        });

    When(/^I create new password "([^"]*)" for "([^"]*)"$/,
        async function(password, userName) {
            userName = userName + global.timestamp;
            let userID;
            const res = await profilePage.isUserExistByFirstName(userName);
            if (res) {
                userID = await profilePage.getUserId(userName);
            }
            if (userID > -1) {
                this.password = await profilePage.setPassword(password, userID);
            }
        });

    When(/^I list user "([^"]*)"$/,
        async (userName) => {
            global.requestJson = {
                headers: {
                    "Content-Type": "application/json",
                    "Authorization": global.authToken
                },
                json: true,
                body: {}
            };
            global.requestJson.resolveWithFullResponse = true;
            global.requestJson.method = "GET";
            global.requestJson.uri = global.runSettings.umsUrl + "/v1/users?username=" + userName;
            global.requestJson.simple = false;
            global.responseJson = await rp(global.requestJson);
        });

    Then(/^password for was changed to "([^"]*)"$/,
        function(password) {
            const res = password === this.password;
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error: ${password} does not match!`).to.be.true;

        });

    When(/^I get list of users$/,
        async function() {
            this.userList = await profilePage.getUsersList().catch((error: any) => { logger.error("getlist error: " + error); });
        });

    Then(/^"([^"]*)" status is "([^"]*)"$/,
        async function(userName, status) {
            if (status !== this.responseJson.body.status) {
                // tslint:disable-next-line: no-unused-expression
                expect(false, `Error: expected status ${status} is not equal to actual status ${this.responseJson.body.status} of user ${userName}`).to.be.true;
            }
        });

    Then(/^user "([^"]*)" appears on list$/,
        async function(userName) {
            userName = userName + global.timestamp;
            const res = profilePage.checkUserExistByFirstName(this.userList, userName);
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error: ${userName} does not exist!`).to.be.true;
        });

    When(/^I create again user name "([^"]*)"$/,
        async function(userName) {
            const duplicateUserName = userName + global.timestamp;
            const user = constants[userName];
            this.responseJson = await profilePage.createUserByAPI(duplicateUserName, user);
            logger.verbose("creating same user again");
        });

    When(/^I search for user "([^"]*)"$/,
        async function(userName) {
            userName = userName + global.timestamp;
            let userID;
            const res = await profilePage.isUserExistByFirstName(userName);
            if (res) {
                userID = await profilePage.getUserId(userName);
            }
            if (userID > -1) {
                this.responseJson = await profilePage.findUserById(userID);
            }
        });

    When(/^I search for user that not exist$/,
        async function() {
            this.userList = await profilePage.getUsersList();
            const lastUser = this.userList.length - 1;
            const userID = String(Number(this.userList[lastUser].id) + 1);
            this.responseJson = await profilePage.findUserById(userID);
        });

    When(/^I update user "([^"]*)"$/,
        async function(userName) {
            const uniqUserName = userName + global.timestamp;
            const user = constants[userName];
            const userID = await profilePage.getUserId(uniqUserName);
            this.responseJson = await profilePage.updateUser(userID, uniqUserName, user);
        });

    When(/^I update user that not exist$/,
        async function() {
            const lastUser = this.userList.length - 1;
            const user = constants.user1
            const userID = String(Number(this.userList[lastUser].id) + 1);
            this.responseJson = await profilePage.updateUser(userID, "notexistuser", user);
        });

    When(/^I delete user name "([^"]*)"$/,
        async function(userName) {
            userName = userName + global.timestamp;
            const userID = await profilePage.getUserId(userName);
            if (userID > -1) {
                this.responseJson = await profilePage.deleteUserById(userID);
            }
        });

    When(/^I filter the data with: "(.*)"$/,
        async (data) => {
            data = constants[data];
            const actualResponse = global.responseJson;
            updateJson(data, "", actualResponse);
            global.responseJson = actualResponse;
        });

    When(/^I find "([^"]*)" users$/,
        async function(userStatus) {
            this.usersList = await profilePage.getUsersListByStatus(userStatus);
        });

    When(/^I search for "([^"]*)"$/,
        async function(searchStringList) {
            this.localSearchStringList = constants[searchStringList];
            this.foundUsers = [];
            for (const searchString of this.localSearchStringList) {
                this.foundUsers.push(await profilePage.searchUsers(searchString));
            }
        });

    Then(/^last login field of user "([^"]*)" was updated correctly$/,
        async function(userName) {
            userName = userName + global.timestamp;
            const userList = await profilePage.searchUsers(userName);
            console.log("stop");
        });

    Then(/^only users that contains "([^"]*)" were found$/,
        async function(searchStringList) {
            let res;
            const listLength = this.foundUsers.length;
            for (let x = 0; x < this.localSearchStringList.length; x++) {
                for (let i = 0; i < listLength; i++) {
                    for (let j = 0; j < this.foundUsers[i].length; j++) {
                        if (!(this.foundUsers[i][j].externalId.includes(this.localSearchStringList[x]))) {
                            res = false;
                            expect(res, `Error: ${this.localSearchStringList[x]} was searched but ${this.foundUsers[i].username} was found`);
                        }
                    }
                }
            }
        });

    Then(/^I expect the response to be: "(.*)"$/,
        (response) => {
            let res;
            const actualResponse = JSON.stringify(global.responseJson);
            const expectedResponse = JSON.stringify(constants[response]).replace(/\s/g, " ");
            const errorMsg = "Actual response is different than expected";
            const diff = jsonDiff.diffString(JSON.parse(actualResponse), JSON.parse(expectedResponse));
            if (diff == "") {
                res = true;
            }
            expect(res, `Error: ${errorMsg}`).to.be.true;
        });

    Then(/^response contains value "([^"]*)"$/,
        async function(data) {
            const actualResponse = JSON.stringify(this.roleList);
            const res = actualResponse.includes(data);
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error: ${data} does not exist!`).to.be.true;
        });

    Then(/^response key "([^"]*)" value is "([^"]*)"$/,
        async (key, value) => {
            const obj = global.responseJson;
            const actualValue = global.responseJson.body[0][key];
            //  let findValue = function findValue(obj) {
            //     for(let localKey in obj){
            //         if(obj.hasOwnProperty(localKey)){
            //             if(localKey === key){
            //                 res = obj[localKey] === value;
            //                 return res;
            //             }
            //             else{
            //                 let val = obj[localKey];
            //                 findValue(val);
            //             }
            //         }
            //     }
            // }

            const res = actualValue === value;
            expect(res, `Error: the value of: ${key} is not: ${value} `).to.be.true;
        });

    Then(/^only "([^"]*)" users are returned$/,
        async function(userStatus) {
            const res = profilePage.checkUsersStatus(this.usersList, userStatus);
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error: not all users are with status:  ${userStatus}`).to.be.true;
        });
});
