
import { defineSupportCode } from "cucumber";
const scenarioWorld = require("../../../node_modules/@cellebrite/automation-wdio-helpers/scenarioWorld");
import {appPage} from "../../api_objects/appPage";
import {permissionPage} from "../../api_objects/permissionPage";
import {rolePage} from "../../api_objects/rolePage";
import {groupPage} from "../../api_objects/groupPage";
import {profilePage} from "../../api_objects/profilePage";
const logger = require("@cellebrite/automation-loggers/loggerModule");
import {expect} from "chai";

declare const global: any;

defineSupportCode(({ Given, When, Then }) => {

    Given (/^I create group name "([^"]*)" role "([^"]*)" app "([^"]*)" perm "([^"]*)"$/,
        async function(groupName, roleName, appName, permName) {

            appName = appName + global.timestamp;
            permName = permName + global.timestamp;
            roleName = roleName + global.timestamp;
            groupName = groupName + global.timestamp;

            await appPage.initApp(appName);
            this.appId = scenarioWorld.appId;
            this.appKey = scenarioWorld.appKey;

            await permissionPage.initPerm(appName, this.appId, this.appKey);
            this.permId = scenarioWorld.permId;

            await rolePage.initRole(roleName, this.permId);
            this.roleId = scenarioWorld.roleId;

            await groupPage.initGroup(groupName, this.roleId);
        });

    When(/^I Search roles for group "([^"]*)"$/,
        async function(groupName) {
            groupName = groupName + global.timestamp;
            const groupID = await groupPage.getGroupId(groupName);
            if (groupID > -1){
                this.roleList = await groupPage.getRolesByGroupId(groupID);
            }
        }
    );

    When(/^I get list of groups$/,
        async function() {
            this.groupList = await groupPage.getGroupsList();
        });

    When(/^I search for group "([^"]*)"$/,
        async function(groupName) {
            groupName = groupName + global.timestamp;
            let groupID;
            const res = await groupPage.isGroupExistByName(groupName);
            if (res) {
                    groupID = await groupPage.getGroupId(groupName);
                }
            if (groupID > -1) {
                    this.responseJson = await groupPage.findGroupById(groupID);
                }
        });

    When(/^I update group "([^"]*)"$/,
        async function(groupName) {
            groupName = groupName + global.timestamp;
            const groupID = await groupPage.getGroupId(groupName);
            if (groupID > -1) {
                this.responseJson = await groupPage.updateGroup(groupID, groupName);
            } else {
                // tslint:disable-next-line: no-unused-expression
                expect(false, `Error: ${groupName} does not exist!`).to.be.true;
            }
        });

    When(/^I add role "([^"]*)" to group "([^"]*)"$/,
        async function(roleName, groupName) {
            roleName = roleName + global.timestamp;
            groupName = groupName + global.timestamp;
            const roleId = await rolePage.getRoleId(roleName);
            const groupId = await groupPage.getGroupId(groupName);
            if (roleId > -1){
                this.responseJson = await groupPage.addRolesToGroup(roleId, groupId);
            }
        });

    When(/^I update role "([^"]*)" to group "([^"]*)"$/,
        async function(roleName, groupName) {
            roleName = roleName + global.timestamp;
            groupName = groupName + global.timestamp;
            const roleId = await rolePage.getRoleId(roleName);
            const groupId = await groupPage.getGroupId(groupName);
            if (roleId > -1){
                this.responseJson = await groupPage.updateRoleToGroup(roleId, groupId);
            }
        });

    When(/^I add user "([^"]*)" to group "([^"]*)"$/,
        async function(userName, groupName) {
            userName = userName + global.timestamp;
            groupName = groupName + global.timestamp;

            const groupId = await groupPage.getGroupId(groupName);
            const userId = await profilePage.getUserId(userName);
            if (groupId > -1 && userId > -1){
                this.responseJson = await groupPage.addUserToGroup(groupId, userId);
            }
        });

    When(/^I delete group "group1"$/,
    async () => {
        // todo
    });

    Then(/^amount of returned "([^"]*)" should be "([^"]*)"$/,
        async function(items, amountOfItems) {
            const res = amountOfItems === String(this.roleList.length);
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error, expected amount of roles : ${amountOfItems} does not match actual amount of roles ${this.roleList.length}`).to.be.true;
        });

    Then(/^The group "([^"]*)" appears on list$/,
        async function(groupName) {
            groupName = groupName + global.timestamp;
            const res = await groupPage.checkGroupExistByName(this.groupList, groupName);
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error: ${groupName} does not exist!`).to.be.true;
        });
});