import { defineSupportCode } from 'cucumber';

var rp = require('request-promise');
import { appPage } from '../../api_objects/appPage';
const scenarioWorld = require('@cellebrite/automation-wdio-helpers/scenarioWorld');
import { permissionPage } from '../../api_objects/permissionPage';
const logger = require('@cellebrite/automation-loggers/loggerModule');
import { expect } from 'chai';
import { permutePath } from 'tough-cookie';
declare const global: any;
declare const constants: any;
defineSupportCode(({ Given, When, Then }) => {

    Given(/^I create permission "([^"]*)" with application "([^"]*)"$/,
        async function(permName, appName) {
            appName = appName + global.timestamp;
            let res = await appPage.isAppExistByName(appName);
            if (!res) {
                await appPage.createAppByAPI(appName);
            }
            const appId = await appPage.getAppId(appName);
            const appKey = await appPage.getAppKey(appName);
            permName = permName + global.timestamp;
            res = await permissionPage.isPermExistByName(permName);
            if (!res) {
                await permissionPage.createPermByAPI(permName, appId, appKey);
            }
        });

    When(/^I get list of permissions$/,
        async function() {
            this.permList = await permissionPage.getPermList().catch((error: string) => { logger.error("getlist error: " + error); });
        });

    When(/^I update permission "([^"]*)" with application "([^"]*)"$/,
        async function(permName, appName) {
            permName = permName + global.timestamp;
            appName = appName + global.timestamp;
            const permId = await permissionPage.getPermId(permName);
            const appId = await appPage.getAppId(appName);
            const appKey = await appPage.getAppKey(appName);
            this.responseJson = await permissionPage.updatePerm(permId, appId, appKey);
        });

    Then(/^permission "([^"]*)" appears on list$/,
        async function(permName) {
            permName = permName + global.timestamp;
            const res = appPage.checkAppExistByName(this.permList, permName);
            // tslint:disable-next-line: no-unused-expression
            expect(res, `Error: ${permName} does not exist!`).to.be.true;
        });

    When(/^I get all permissions for auth user with app$/,
        async function() {
            this.responseJson = await permissionPage.getAllPermission(this.appKey, this.uniqueUserName);
        });

    Then(/^permissions should match$/,
        function() {
            // tslint:disable-next-line: no-unused-expression
            expect(this.responseJson[0].id ==  this.permId, `Error: permissions does not match`).to.be.true;
        });
});

