const logger = require('@cellebrite/automation-loggers/loggerModule');
const reportPortal = require('@cellebrite/automation-report-portal/reportPortal');
const countMyPickles = require("@cellebrite/automation-wdio-helpers/picklesCounter");

let umsUrlIndex = process.argv.indexOf('umsUrl');
let umsUrl = (umsUrlIndex > -1) ? process.argv[umsUrlIndex + 1] : undefined;

let envNameIndex = process.argv.indexOf('envName');
let envName = (envNameIndex > -1) ? process.argv[envNameIndex + 1] : undefined;

if (umsUrl === undefined) {
    logger.error("Run failed: Didn't receive any ums URL!");
    process.exit(1);
    
}

if (envName === undefined) {
    logger.error("Run failed: Didn't receive any ums environment name!");
    logger.error("using UMS instead");
}
logger.verbose("Received ums URL: " + umsUrl);
logger.verbose('Received ums Env Name' + envName);

if (process.env.automationRunTags == null)
{
    logger.verbose('No env defined tags. Running @debug scenarios...')
    process.env.automationRunTags = '@debug'
}

let featureFolder = './automation/src/features/';
let tagExpression = process.env.automationRunTags;
countMyPickles(featureFolder, tagExpression);


//#########################################
reportPortal.startLaunch("UMS",envName).then((res) => {
    if (res !== null) {
        process.env.launchId = res.id;
        logger.info(`Report portal launch ID: ${res.id}`);
    }
    var Launcher = require('webdriverio').Launcher;
    var wdio = new Launcher('automation/wdio.conf.debug.multiple.js', {umsUrl,cucumberOpts: {tagExpression: process.env.automationRunTags}, ignoreUndefinedDefinitions: false, strict: true});
    const {spawnSync} = require('child_process');
    var ssrs =require('@cellebrite/automation-reports/ssrs');
    const timestamp = new Date().toISOString();      

    if (process.env.automationSessionId == null)
    {
        logger.verbose('Setting up the run timestamp')
        const moment = require('moment');
        const time = moment();
        var time_format = time.format('YYYYMMDD-HHmmssSSS');        
        process.env.automationSessionId = time_format;
    }

    logger.verbose('Session selected tags: ' + process.env.automationRunTags)
    logger.verbose('Automation session ID: ' + process.env.automationSessionId);
    
    wdio.run().then((code) => {            
        reportPortal.getLaunch().then((res) => {
            if (res !== null) {
                let errMsg = "Number of executed scenarios was different than expected";
                let totalExecutedScenarios = res.statistics.executions.total;                        
                logger.info("Number of total executed scenarios: " + totalExecutedScenarios);
                logger.info("Number of passed scenarios: " + res.statistics.executions.passed);
                logger.info("Number of failed scenarios: " + res.statistics.executions.failed);
                logger.info("Number of supposed to run scenarios: " + process.env.totalScenarioCount);   
                var result = spawnSync('node', ['./automation/node_modules/@cellebrite/automation-reports/generateHtmlReport.js'],{ shell: true });            
                if (totalExecutedScenarios !== process.env.totalScenarioCount) {
                    logger.error(`Number of executed scenarios (${totalExecutedScenarios}) is different than expected (${process.env.totalScenarioCount})`);
                    reportPortal.finishLaunch(process.env.launchId).then(() => process.exit(1));                    
                } else {
                    reportPortal.finishLaunch(process.env.launchId);
                    ssrs.updateSql("automation/json-report-4sql/"+process.env.automationSessionId,process.env.automationSessionId).then(()=>{                    
                        process.exit(code);
                    })
                }                     
            }            
        });
    }, (error) => {
            logger.error('Launcher failed to start the test', error.stacktrace);
            process.exit(1);
        });
});


