const automationHooks = require('./node_modules/@cellebrite/automation-wdio-helpers/automationHooks');
const umsHooks = require('./src/common/hooks/umsHooks');

exports.config = {
    debug: false,
    //execArgv: ['--inspect=127.0.0.1:5859'],    

    suites: {
        orgStructure: [
            //'./automation/src/features/**/District.feature',
            //'./automation/src/features/**/Unit.feature',
           // './automation/src/features/**/caseDetails.feature'
        ],
    },
    specs: [
         './automation/src/features/**/*.feature',
    ],

    maxInstances: 1,

    capabilities: [{
        maxInstances: 1,
        browserName: 'chrome',
    }],

    sync: true,

    logLevel: 'error',

    coloredLogs: true,

    screenshotPath: './automation/errorShots/',

    baseUrl: 'http://localhost:8080',

    waitforTimeout: 10000,

    connectionRetryTimeout: 90000,

    connectionRetryCount: 3,

    services: ['selenium-standalone'],
    host: 'selenium-cluster.dev.local',
    path: '/wd/hub',

    framework: 'cucumber',

    reporters: ['spec', 'dot', 'junit'],
    reporterOptions: {
        junit:{
            outputDir: './automation/junit-report',
        }
    },

    cucumberOpts: {
        backtrace: false,
        compiler: [
            'js:babel-register',
        ],
        failAmbiguousDefinitions: true,
        failFast: false,
        ignoreUndefinedDefinitions: false,
        name: [],
        snippets: true,
        source: true,
        profile: [],
        require: [
            './automation/src/steps/*.*'            
        ],
        snippetSyntax: undefined,
        strict: true,
        tagsInTitle: true,
        timeout: 600000,
    },


    beforeSession: async (config, capabilities, specs) => {             
        this.umsUrl = config.umsUrl;
        await automationHooks.beforeSessionTasks(this.screenshotPath);
    },  

    before: async () => {        
        await automationHooks.beforeTasks();        
        await umsHooks.beforeTasks(this.umsUrl);
    },

    beforeScenario: async (scenario) => {
        await automationHooks.beforeScenarioTasks(this.screenshotPath, scenario, browser);
    },

    afterScenario: async (scenarioResult) => {
        await automationHooks.afterScenarioTasks();
    },

    beforeStep: async (step) => {
        await automationHooks.beforeStepTasks(step);
    },

    afterStep: async (stepResult) => {
        await automationHooks.afterStepTasks(stepResult, browser);
    },

    beforeFeature: async (feature) => {
        await automationHooks.beforeFeatureTasks(feature);
    },

    afterFeature: async (featureResult) => {
        await automationHooks.afterFeatureTasks(featureResult);
    },
};
