const automationHooks = require('./node_modules/@cellebrite/automation-wdio-helpers/automationHooks');
const umsHooks = require('./src/common/hooks/umsHooks');

exports.config = {
    
    debug: false,
    execArgv: ['--inspect=127.0.0.1:5859'],

    specs: JSON.parse(process.env.featuresToRun),

    capabilities: [{
        maxInstances: 1,
        browserName: 'chrome',
        chromeOptions: {
            args: [//"--start-maximized"
             "--headless",// "--disable-gpu"               
            ],
        },
    }],

    sync: true,

    logLevel: 'error', // options: 'silent', 'command', 'result', 'data', 'error', 'verbose'

    coloredLogs: true,

    screenshotPath: './automation/errorShots/',

    baseUrl: 'http://localhost:8080',

    waitforTimeout: 10000,

    connectionRetryTimeout: 90000,

    connectionRetryCount: 3,

    // services: ['selenium-standalone'], // ATTENTION: A local LDAP, which runs on JAVA, might collide with the selenium-standalone. use cluster instead.   
    
    //un-comment "host" and "path" in order to use a cluster for parallel run. if so, comment "services: ['selenium-standalone']"
    host: 'selenium-cluster.dev.local',
    path: '/wd/hub',

    framework: 'cucumber',

    reporters: ['spec', 'dot', 'junit'],
    reporterOptions: {
        junit:{
            outputDir: './automation/junit-report',
        }
    },

    cucumberOpts: {
        dryRun: false,
        backtrace: false,
        compiler: ['ts:ts-node/register'],
        failAmbiguousDefinitions: true,
        failFast: false,
        ignoreUndefinedDefinitions: false,
        name: [],
        snippets: true,
        source: true,
        profile: [],
        require: ['./automation/src/steps/**/*.ts'],
        snippetSyntax: undefined,
        strict: true,
        tagsInTitle: true,
        timeout: 600000,
    },

    

    beforeSession: async (config, capabilities, specs) => {        
        await automationHooks.beforeSessionTasks(this, config);     
        umsHooks.beforeSessionTasks(this)     
    },  

    before: async () => {
        await automationHooks.beforeTasks(this);
        umsHooks.beforeTasks(this);
    },

    beforeScenario: async (scenario) => {
        await automationHooks.beforeScenarioTasks(this.config.screenshotPath, scenario, browser);
        umsHooks.beforeScenarioTasks();
    },

    afterScenario: async (scenarioResult) => {
        await automationHooks.afterScenarioTasks();
    },

    beforeStep: async (step) => {
        await automationHooks.beforeStepTasks(step);
    },

    afterStep: async (stepResult) => {
        await automationHooks.afterStepTasks(this, stepResult, browser);
    },

    beforeFeature: async (feature) => {
        await automationHooks.beforeFeatureTasks(feature);
        await umsHooks.beforeFeatureTasks(feature);
    },

    afterFeature: async (featureResult) => {
        await automationHooks.afterFeatureTasks(featureResult);
    },

    afterSession: async () => {
        await automationHooks.afterSessionTasks();
    }
};

