const automationHooks = require('@cellebrite/automation-wdio-helpers/automationHooks');
exports.config = {
    debug: true,
    execArgv: ['--inspect=127.0.0.1:5859'],

    // suites: {
    //    login: [
    //        './automation/src/ui/features/**/*.feature',
    //    ],
    // },

    specs: [
         './automation/src/api/features/**/*.feature',
         //'./automation/src/ui/features/**/automationGeneral2.feature',
    ],

    maxInstances: 1,

    capabilities: [{
        maxInstances: 1,
        browserName: 'chrome',
        chromeOptions: {
            args: ["--start-maximized", 
                // '--test-type', 'show-fps-counter=true', '--web-security=no spec.js', '--use-fake-device-for-media-stream', '--use-fake-ui-for-media-stream'
            ],
        },
    }],

    sync: true,

    logLevel: 'error',

    coloredLogs: true,

    screenshotPath: './automation/errorShots/',

    baseUrl: 'http://localhost:8080',

    waitforTimeout: 10000,

    connectionRetryTimeout: 90000,

    connectionRetryCount: 3,

    services: ['selenium-standalone'],

    framework: 'cucumber',

    reporters: ['spec', 'dot', 'junit'],
    reporterOptions: {
        junit:{
            outputDir: './automation/junit-report',
        }
    },

    cucumberOpts: {
        backtrace: false,
        compiler: [
            'js:babel-register',
        ],
        failAmbiguousDefinitions: true,
        failFast: false,
        ignoreUndefinedDefinitions: false,
        name: [],
        snippets: true,
        source: true,
        profile: [],
        require: [
            './automation/src/api/steps/*.js',           
        ],
        snippetSyntax: undefined,
        strict: true,
        tagsInTitle: true,
        timeout: 600000,
    },


    beforeSession: async (config, capabilities, specs) => {
        await automationHooks.beforeSessionTasks(this.screenshotPath);
    },  

    before: async () => {
        await automationHooks.beforeTasks();
    },

    beforeScenario: async (scenario) => {
        await automationHooks.beforeScenarioTasks(this.screenshotPath, scenario, browser);
    },

    afterScenario: async (scenarioResult) => {
        await automationHooks.afterScenarioTasks();
    },

    beforeStep: async (step) => {
        await automationHooks.beforeStepTasks(step);
    },

    afterStep: async (stepResult) => {
        await automationHooks.afterStepTasks(stepResult, browser);
    },

    beforeFeature: async (feature) => {
        await automationHooks.beforeFeatureTasks(feature);
    },

    afterFeature: async (featureResult) => {
        await automationHooks.afterFeatureTasks(featureResult);
    },
};
